#include "cve.h"

/**
 * Applies the requested kernel element-wise.
 *
 * @param A matrix to apply the kernel to.
 * @param h bandwidth parameter.
 * @param kernel the kernel to be used.
 * @param B (in/out) matrix `A` with element-wise applied kernel.
 *
 * @returns ether the passed `B`, or a new created matrix if `B` was NULL.
 */
mat* applyKernel(const mat* A, const double h, kernel kernel, mat* B) {
    int i, nn = A->nrow * A->ncol;
    int nn4 = 4 * (nn / 4);
    double scale;
    const double * restrict a;
    double * restrict b;

    if (!B) {
        B = matrix(A->nrow, A->ncol);
    } else if (nn != B->nrow * B->ncol) {
        // TODO: error handling!
    }

    a = A->elem;
    b = B->elem;
    switch (kernel) {
        case gauss:
            scale = -0.5 / (h * h);
            for (i = 0; i < nn4; i += 4) {
                b[i]     = exp(scale * a[i]     * a[i]);
                b[i + 1] = exp(scale * a[i + 1] * a[i + 1]);
                b[i + 2] = exp(scale * a[i + 2] * a[i + 2]);
                b[i + 3] = exp(scale * a[i + 3] * a[i + 3]);
            }
            for (; i < nn; ++i) {
                b[i] = exp(scale * a[i] * a[i]);
            }
            break;
        default:
            // TODO: error handling!
            break;
    }

    return B;
}

/**
 * Scaling matrix `S` defined as
 *      s_{i j} = (L_i - (Y_j - y1_i)^2) * d_{i j} * w_{i j}
 *
 * Mapping of vectors `L`, `y1` and `Y` combinations.
 *
 *                                       Y[j]
 *                                  ------ j ----->
 *                  s[0]    s[n]  . . .  s[jn]  . . .  s[(j-1)n]
 *                  s[1]   s[n+1] . . . s[jn+1] . . . s[(j-1)n+1]
 *               |    .       .   .        .    .          .
 *               |    .       .     .      .      .        .
 *                    .       .       .    .        .      .
 *  L[i], y1[i]  i  s[i]   s[n+i] . . . s[jn+i] . . . s[(j-1)n+i]
 *                    .       .   .        .    .          .
 *               |    .       .     .      .      .        .
 *               v    .       .       .    .        .      .
 *                 s[n-1] s[2n-1] . . .  s[n-1] . . .   s[nn-1]
 *
 * @param L per sample loss vector of (lenght `n`).
 * @param Y responces (lenght `n`).
 * @param y1 weighted responces (lenght `n`).
 * @param D distance matrix (dim. `n x n`).
 * @param W weight matrix (dim. `n x n`).
 * @param kernel the kernel to be used.
 *
 * @returns passed matrix `S` if not NULL, then a new `n x n` matrix is created.
 *
 * @example Basically equivalent to the following R function.
 * r_LS <- function(L, Y, y1, D, W) {
 *     # get dimension
 *     n <- length(L)
 *     # Indices
 *     i <- rep(1:n, n)
 *     j <- rep(1:n, each = n)
 *     # Compute S
 *     matrix(L[i] - (Y[j] - y1[i])^2, n) * W * D
 * }
 * 
 * @details mapping for indices used in blocked implementation.
 *
 * n ..... Dimensions of `S`, a `n x n` matrix.
 * B ..... BLOCK_SIZE
 * rB .... block reminder, aka rB = B % n.
 *
 *                                    Y[j]
 *                            0  -----  j  ----->  n
 *                          0 +--------------------+
 *                            | '                  |
 *                          ' | k      B x n       |
 *                          ' | v                  |
 *                          ' +--------------------+
 *            L[i], y1[i]   i | '                  |
 *                          ' | k      B x n       |
 *                          ' | v                  |
 *                          v +--------------------+
 *                            | k     rB x n       |
 *                          n +--------------------+
 */
// TODO: fix: cache misses in Y?!
mat* adjacence(const mat *vec_L, const mat *vec_Y, const mat *vec_y1,
               const mat *mat_D, const mat *mat_W, kernel kernel,
               mat *mat_S) {
    int i, j, k, n = vec_L->nrow;
    int block_size, block_batch_size;
    int max_size = 64 < n ? 64 : n; // Block Size set to 64

    double Y_j, tmp0, tmp1, tmp2, tmp3;
    double *Y  = vec_Y->elem;
    double *L  = vec_L->elem;
    double *y1 = vec_y1->elem;
    double *D, *W, *S;

    // TODO: Check dims.

    if (!mat_S) {
        mat_S = matrix(n, n);
    }

    for (i = 0; i < n; i += block_size) {
        /* Set blocks (left upper corner) */
        S = mat_S->elem + i;
        D = mat_D->elem + i;
        W = mat_W->elem + i;
        /* determine block size */
        block_size = n - i;
        if (block_size > max_size) {
            block_size = max_size;
        }
        block_batch_size = 4 * (block_size / 4);
        /* for each column in the block */
        for (j = 0; j < n; ++j, S += n, D += n, W += n) {
            Y_j = Y[j];
            /* iterate over block rows */
            for (k = 0; k < block_batch_size; k += 4) {
                tmp0 = Y_j - y1[k];
                tmp1 = Y_j - y1[k + 1];
                tmp2 = Y_j - y1[k + 2];
                tmp3 = Y_j - y1[k + 3];
                S[k]     = (L[k]     - (tmp0 * tmp0)) * D[k]     * W[k];
                S[k + 1] = (L[k + 1] - (tmp1 * tmp1)) * D[k + 1] * W[k + 1];
                S[k + 2] = (L[k + 2] - (tmp2 * tmp2)) * D[k + 2] * W[k + 2];
                S[k + 3] = (L[k + 3] - (tmp3 * tmp3)) * D[k + 3] * W[k + 3];
            }
            for (; k < block_size; ++k) {
                tmp0 = Y_j - y1[k];
                S[k] = (L[k] - (tmp0 * tmp0)) * D[k] * W[k];
            }
        }
        L  += block_size;
        y1 += block_size;
    }

    return mat_S;
}

// int getWorkLen(const int n, const int p, const int q) {
//     int mpq; /**< Max of p and q */
//     int nn = ((n - 1) * n) / 2;

//     if (p > q) {
//         mpq = p;
//     } else {
//         mpq = q;
//     }
//     if (nn * p < (mpq + 1) * mpq) {
//         return 2 * (mpq + 1) * mpq;
//     } else {
//         return (nn + mpq) * mpq;
//     }
// }

// double cost(const unsigned int method,
//             const int n,
//             const double *Y,
//             const double *vecK,
//             const double *colSums,
//             double *y1, double *L) {
//     int i, j, k;
//     double tmp, sum;

//     for (i = 0; i < n; ++i) {
//         y1[i] = Y[i];
//         L[i] = Y[i] * Y[i];
//     }

//     for (k = j = 0; j < n; ++j) {
//         for (i = j + 1; i < n; ++i, ++k) {
//             y1[i] += Y[j] * vecK[k];
//             y1[j] += Y[i] * vecK[k];
//             L[i] += Y[j] * Y[j] * vecK[k];
//             L[j] += Y[i] * Y[i] * vecK[k];
//         }
//     }

//     for (i = 0; i < n; ++i) {
//         y1[i] /= colSums[i];
//         L[i] /= colSums[i];
//     }

//     tmp = 0.0;
//     if (method == CVE_METHOD_WEIGHTED) {
//         sum = 0.0;
//         for (i = 0; i < n; ++i) {
//             tmp += (colSums[i] - 1.0) * (L[i] -= y1[i] * y1[i]);
//             sum += colSums[i];
//         }
//         return tmp / (sum - (double)n); // TODO: check for division by zero!
//     } else {
//         for (i = 0; i < n; ++i) {
//             tmp += (L[i] -= y1[i] * y1[i]);
//         }
//         return tmp / (double)n;
//     }
// }

// void scaling(const unsigned int method,
//              const int n,
//              const double *Y, const double *y1, const double *L,
//              const double *vecD, const double *vecK,
//              const double *colSums,
//              double *vecS) {
//     int i, j, k, nn = (n * (n - 1)) / 2;
//     double tmp;

//     if (method == CVE_METHOD_WEIGHTED) {
//         for (k = j = 0; j < n; ++j) {
//             for (i = j + 1; i < n; ++i, ++k) {
//                 tmp = Y[j] - y1[i];
//                 vecS[k]  = (L[i] - (tmp * tmp));
//                 tmp = Y[i] - y1[j];
//                 vecS[k] += (L[j] - (tmp * tmp));
//             }
//         }
//     } else {
//         for (k = j = 0; j < n; ++j) {
//             for (i = j + 1; i < n; ++i, ++k) {
//                 tmp = Y[j] - y1[i];
//                 vecS[k]  = (L[i] - (tmp * tmp)) / colSums[i];
//                 tmp = Y[i] - y1[j];
//                 vecS[k] += (L[j] - (tmp * tmp)) / colSums[j];
//             }
//         }
//     }

//     for (k = 0; k < nn; ++k) {
//         vecS[k] *= vecK[k] * vecD[k];
//     }
// }
